<?php

namespace Drupal\webform_myemma\Plugin\WebformHandler;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;
use Drupal\webform\WebformSubmissionConditionsValidatorInterface;
use Drupal\webform\WebformTokenManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use MarkRoland\Emma\Client;

/**
 * Form submission to MyEmma handler.
 *
 * @WebformHandler(
 *   id = "myemma",
 *   label = @Translation("MyEmma"),
 *   category = @Translation("MyEmma"),
 *   description = @Translation("Sends a form submission to a MyEmma group."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_UNLIMITED,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 * )
 */
class WebformMyEmmaHandler extends WebformHandlerBase {

  /**
   * The token manager.
   *
   * @var \Drupal\webform\WebformTranslationManagerInterface
   */
  protected $tokenManager;

  /**
   * The plugin configuration.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $emmaConfig;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, LoggerChannelFactoryInterface $logger_factory, ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager, WebformSubmissionConditionsValidatorInterface $conditions_validator, WebformTokenManagerInterface $token_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $logger_factory, $config_factory, $entity_type_manager, $conditions_validator);
    $this->tokenManager = $token_manager;
    $this->emmaConfig = $config_factory->get('webform_myemma.settings');
    $this->logger = $logger_factory->get('webform_myemma');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('logger.factory'),
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('webform_submission.conditions_validator'),
      $container->get('webform.token_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'email' => '',
      'group_id' => '',
      'fields' => [],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $this->applyFormStateToConfiguration($form_state);

    $form['myemma'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('MyEmma settings'),
    ];
    $form['myemma']['group_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Group Id'),
      '#required' => TRUE,
      '#default_value' => $this->configuration['group_id'],
      '#description' => $this->t('Group ID (numerical) of the list to add the submission to.'),
      '#size' => 60,
      '#maxlength' => 128,
    ];

    $fields = $this->getWebform()->getElementsDecoded();
    $options = [];
    $options[''] = $this->t('- Select an option -');
    foreach ($fields as $field_name => $field) {
      if ($field['#type'] == 'email') {
        $options[$field_name] = $field['#title'];
      }
    }

    $form['myemma']['email'] = [
      '#type' => 'select',
      '#title' => $this->t('Email field'),
      '#required' => TRUE,
      '#default_value' => $this->configuration['email'],
      '#options' => $options,
    ];

    // Get elements options.
    $element_options = [];
    $elements = $this->webform->getElementsInitializedFlattenedAndHasValue();
    $element_key = [];

    foreach ($elements as $element_key => $element) {
      if($element_key != 'email'){
        $element_options[$element_key] = (isset($element['#title'])) ? $element['#title'] : $element_key;
      }

    }


    $form['myemma']['fields'] = [
      '#type' => 'webform_mapping',
      '#title' => 'MyEmma Field Shortcut',
      '#description' => $this->t('Shortcut name from MyEmma, contains underscores.'),
      '#description_display' => 'before',
      '#default_value' => $this->configuration['fields'],
      '#source' => $element_options,
    ];

    return $form;
  }
  /**
   * Display form fields for mapping to the MyEmma fields.
   *
   * @param array &$form
   *   The form array.
   * @param Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function selectLookup(array &$form, FormStateInterface $form_state) {
    $lookup_by = $form_state->getValue('map_to_webform');
    $form = [];
    switch ($lookup_by) {
      case '1':
        $form['myemma']['fields'] = [
          '#type' => 'webform_mapping',
          '#title' => 'My Emma Field Shortcut',
          '#description' => $this->t('Please select which fields webform submission data should be mapped to'),
          '#description_display' => 'before',
          '#default_value' => $this->configuration['fields'],
          '#required' => TRUE,
          '#source' => $element_options,
        ];
        break;
      default:
        break;
      }
    }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $values = $form_state->getValues();
    foreach ($this->configuration as $name => $value) {
      if (isset($values['myemma'][$name])) {
        $this->configuration[$name] = $values['myemma'][$name];
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(WebformSubmissionInterface $webform_submission, $update = TRUE) {
    // If update, do nothing
    if ($update) {
      return;
    }

    $fields = $webform_submission->toArray(TRUE);

    // Replace tokens.
    $configuration = $this->tokenManager->replace($this->configuration, $webform_submission);

    $email = $fields['data'][$configuration['email']];
    $emma_fields = [];
    foreach($configuration['fields'] as $key => $emma_name){
      foreach($fields['data'] as $name => $value){
          if($key == $name){
            $emma_fields[$emma_name] = $fields['data'][$name];
          }
        }

    }
    
    // Submit to MyEmma
    try {
      $emmaClient = new Client($this->emmaConfig->get('account_id'), $this->emmaConfig->get('public_key'), $this->emmaConfig->get('private_key'));

      $response = $emmaClient->import_single_member($email, $emma_fields, [$configuration['group_id']]);

      if(!$response) {
        $this->logger->error('MyEmma settings are incorrect, set at /admin/config/services/webform_myemma and confirm the Group ID is correct in webform handler');
      }
    }
    catch (\Exception $e){
      $this->logger->error('MyEmma settings not set, set at /admin/config/services/webform_myemma');
    }
  }

}
